use std::any::Any;

pub trait AsAny {
    fn as_any(&self) -> &dyn Any;
}

impl<T: Any> AsAny for T {
    fn as_any(&self) -> &dyn Any {
        self
    }
}

pub trait IntoWeak<T> {
    fn as_weak(self) -> std::rc::Weak<T>;
}

impl<T> IntoWeak<T> for std::rc::Rc<T> {
    fn as_weak(self) -> std::rc::Weak<T> {
        std::rc::Rc::downgrade(&self)
    }
}

impl<'a, T> IntoWeak<T> for &'a std::rc::Rc<T> {
    fn as_weak(self) -> std::rc::Weak<T> {
        std::rc::Rc::downgrade(&self)
    }
}

impl<T> IntoWeak<T> for std::rc::Weak<T> {
    fn as_weak(self) -> std::rc::Weak<T> {
        self
    }
}

impl<'a, T> IntoWeak<T> for &'a std::rc::Weak<T> {
    fn as_weak(self) -> std::rc::Weak<T> {
        self.clone()
    }
}
